
typedef struct mySATURN
{
	char dn;
	char up;
	char lf;
	char rt;
	char a, b, c;
	char x, y, z;
	char start;
	char r,l;
} mySATURN;

void ScanSaturn( mySATURN *saturn, int base )
{
	int s0, s1, s2, s3;

	_outp(base, 255-(2+1)); // Power up, select=00
	s0 = _inp(base+1);
	_outp(base, 255-(2+0)); // Power up, select=01
	s1 = _inp(base+1);
	_outp(base, 255-(0+1)); // Power up, select=10
	s2 = _inp(base+1);
	_outp(base, 255-(0+0)); // Power up, select=11
	s3 = _inp(base+1);
	_outp(base, 0); // Power down joystick
	
	saturn->r = (s0&8)?1:0;
	saturn->x = (s0&16)?1:0;
	saturn->y = (s0&32)?1:0;
	saturn->z = (s0&64)?1:0;

	saturn->start = (s1&8)?1:0;
	saturn->a = (s1&16)?1:0;
	saturn->c = (s1&32)?1:0;
	saturn->b = (s1&64)?1:0;

	saturn->rt = (s2&8)?1:0;
	saturn->lf = (s2&16)?1:0;
	saturn->dn = (s2&32)?1:0;
	saturn->up = (s2&64)?1:0;

	saturn->l = (s0&8)?1:0;
}


typedef struct myGENESIS
{
	char dn;
	char up;
	char lf;
	char rt;
	char a, b, c;
	char start;
	char x, y, z, md;
} myGENESIS;

void ScanGenesis( myGENESIS *genesis, int base, int sixbutton )
{
	int s0, s1, s2, c, c2;

	_outp(base+2, 0x0c);
	_outp(base, 254); // Power up, select=0
	s0 = _inp(base+1);
	_outp(base, 255); // Power up, select=1
	s1 = _inp(base+1);
	c = _inp(base+2);
	
	if (sixbutton)
	{
		_outp(base, 254); // sel=0;
		_outp(base, 255); // sel=1;
		_outp(base, 254); // sel=0;
		_outp(base, 255); // sel=1;
		s2=_inp(base+1);
		c2=_inp(base+2);
	}
	_outp(base, 0); // Power down joystick
	
	genesis->dn = (c&2)?1:0;
	genesis->up = (c&1)?1:0;
	genesis->lf = (s1&64)?0:1;
	genesis->rt = (s1&128)?1:0;
	genesis->a = (s0&32)?0:1;
	genesis->b = (s1&32)?0:1;
	genesis->c = (s1&16)?0:1;
	genesis->start = (s0&16)?0:1;
	if (sixbutton)
	{
		genesis->z = (c2&1)?1:0;
		genesis->y = (c2&2)?1:0;
		genesis->x = (s2&64)?0:1;
		genesis->md = (s2&128)?1:0;
	}
}

typedef struct myATARI
{
	char up;
	char dn;
	char lf;
	char rt;
	char btn;
} myATARI;

void ScanAtari( myATARI *atari, int base )
{
	int c;

	//	Atari		IBM
	//up	1		1	(c0-)
	//dn	2		14	(c1-)
	//lf	3		16	(c2+)
	//rt	4		17	(c3-)
	//btn	6		11	(s7-)  (connect 11 to 4p7R to 2 (d0), drive d0 1)
	//gnd	8		18
	_outp(base+2, 4); //All C's high, disconnected
	_outp(base, 1); // Power joy button sensor
	atari->btn = (_inp(base+1) & 128)?0:1;
	_outp(base, 0); // Power down joy button sensor
	c=_inp(base+2);
	atari->up = (c&1)?1:0;
	atari->dn = (c&2)?1:0;
	atari->lf = (c&4)?0:1;
	atari->rt = (c&8)?1:0;
}

typedef struct mySNES
{
	char dn;
	char up;
	char lf;
	char rt;
	char a, b;
	char x, y;
	char l, r;
	char sel, start;
} mySNES;

#define SNES_PWR (128+64+32+16+8)
#define SNES_CLK 1  // base+0
#define SNES_LAT 2  // base+0
#define SIN (negate?((_inp(base+1)&snes_din)?1:0):((_inp(base+1)&snes_din)?0:1))

void ScanSNES(mySNES *snes, int base, int snes_din, int negate)
{
	_outp(base, SNES_PWR+SNES_CLK); // Power up!
	_outp(base, SNES_PWR+SNES_LAT+SNES_CLK); // Latch it!

	_outp(base, SNES_PWR+SNES_CLK);
	snes->b = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->y = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->sel = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->start = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->up = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->dn = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->lf = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->rt = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->a = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->x = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->l = SIN;
	_outp(base, SNES_PWR);
	_outp(base, SNES_PWR+SNES_CLK);
	snes->r = SIN;
	_outp(base, 0); // Power it down
}

typedef struct JAG
{
	char dn;
	char up;
	char lf;
	char rt;
	char a, b, c, d;
	char pause;
	char opt;
	char kp[12];
} JAG;

void ScanJag(JAG *jag, int base)
{
	int r4, r4p, r3, r2, r1;
	_outp(base, 255-8);
	_outp(base+2, 4);
	r4 = _inp(base+1);
	jag->dn = (r4 & 16)?0:1;
	jag->up = (r4 & 8)?0:1;
	jag->lf = (r4 & 32)?0:1;
	jag->rt = (r4 & 128)?1:0;
	jag->a = (r4 & 64)?0:1;
	r4p = _inp(base+2);
	jag->pause = (r4p&2)?1:0;
	_outp(base, 255-4);
	r3 = _inp(base+1);
	jag->b = (r3 & 64)?0:1;
	jag->kp[1] = (r3 & 128)?1:0;
	jag->kp[4] = (r3 & 32)?0:1;
	jag->kp[7] = (r3 & 16)?0:1;
	jag->kp[10]= (r3 & 8)?0:1;
	_outp(base, 255-2);
	r2 = _inp(base+1);
	jag->c = (r2 & 64)?0:1;
	jag->kp[2] = (r2 & 128)?1:0;
	jag->kp[5] = (r2 & 32)?0:1;
	jag->kp[8] = (r2 & 16)?0:1;
	jag->kp[0] = (r2 & 8)?0:1;
	_outp(base, 255-1);
	r1 = _inp(base+1);
	jag->opt = (r1 & 64)?0:1;
	jag->kp[3] = (r1 & 128)?1:0;
	jag->kp[6] = (r1 & 32)?0:1;
	jag->kp[9] = (r1 & 16)?0:1;
	jag->kp[11]= (r1 & 8)?0:1;
}


int PollStick()
{
#ifdef JAGUAR
	JAG jag;
#endif
#ifdef SNES
	mySNES snes;
	int din[5] = {64,32,16,8,128};
	int negate[5]={0, 0, 0,0,  1};
#endif
#ifdef ATARI
	myATARI atari;
#endif
#ifdef GENESIS
	myGENESIS genesis;
#endif
#ifdef SATURN
	mySATURN saturn;
#endif

#ifdef JAGUAR
	ScanJag(&jag, BASEPORT);
#endif

#ifdef SNES
	ScanSNES(&snes, BASEPORT, din[pd->id], negate[pd->id]);
#endif

#ifdef ATARI
	ScanAtari(&atari, BASEPORT);
#endif

#ifdef GENESIS
#ifdef SIXBUTTON
	ScanGenesis(&genesis, BASEPORT, 1);
#else
	ScanGenesis(&genesis, BASEPORT, 0);
#endif
#endif

#ifdef SATURN
	ScanSaturn(&saturn, BASEPORT);
#endif
}

